/*-------------------------------------------------------------------*/
/* PARAMETR.C      Pasi Franti                                       */
/*                 Timo Kaukoranta                                   */
/* Konsultaatioapua antoi:                                           */
/*                 Harri Tarvainen                                   */
/*                                                                   */
/* - Parameter handling.                                             */
/* - Version 0.2: 14.11.1994                                         */
/*                                                                   */
/*-------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "cbshowpa.h"


/* --------------------------- Data structure ---------------------------- */


enum    ParameterType    { BOOL=0, INT, ENUM, STRING };
enum    Boolean          { NO=0, YES=1 };
enum    Special          { NOT_DEFINED=-1 };

typedef struct
        {
        char*  Documentation;
        char   Key;
        int    Number;
        int    Type;
        int    Value;
        int    Min;
        int    Max;
        int    Default;
        int    SaveToFile;
        char*  Name[10];
        int    PrintingCondition;
        } ParameterList;


/* --------------------------- Local constant --------------------------- */


#define MAX_PARAMETER  (Pseudo_Last + 1)


/* ---------------- Parameter data structure definition ----------------- */


extern ParameterList Parameter[ MAX_PARAMETER ];


/* ---------------- Parameter data structure definition ----------------- */


#define Fact( id, doc, a1, a2, a3, a4, a5, a6, a7, a8, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, pr ) \
            { doc, a1, a2, a3, a4, a5, a6, a7, a8,          \
              { n1, n2, n3, n4, n5, n6, n7, n8, n9, n10 },  \
              NO },

ParameterList Parameter[ MAX_PARAMETER ] =
     {
       { " ", ' ', 0, 0, 0, 0, 0, 0, 0, { NULL }, NO },
       #include FACTFILE
       { " ", ' ', 0, 0, 0, 0, 0, 0, 0, { NULL }, NO }
     };

#undef Fact


/* ---------------- Printing condition initialization  ----------------- */


#define Fact( id, doc, a1, a2, a3, a4, a5, a6, a7, a8, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, pr ) \
              Parameter[ _##id ].PrintingCondition = pr;

extern void InitializePrintingConditions()
{
#include FACTFILE
}

#undef Fact


/* ========================  I N T E R F A C E  ========================= */


extern char* Document_(int x)
{
return( Parameter[ x ].Documentation );
}


extern char Key_(int x)
{
return( Parameter[ x ].Key );
}


extern int Number_(int x)
{
return( Parameter[ x ].Number );
}


extern int Type_(int x)
{
return( Parameter[ x ].Type );
}


extern int Value_(int x)
{
return( Parameter[ x ].Value );
}


extern void SetValue_(int x, int v)
{
Parameter[ x ].Value = v;
}


extern int Max_(int x)
{
return( Parameter[ x ].Max );
}


extern int Min_(int x)
{
return( Parameter[ x ].Min );
}


extern int Default_(int x)
{
return( Parameter[ x ].Default );
}


extern int SaveToFile_(int x)
{
return( Parameter[ x ].SaveToFile );
}


extern char* NameOfValue_(int x, int v)
{
return( Parameter[ x ].Name[ v ] );
}


int ShallWePrint_(int x)
{
return( Parameter[ x ].PrintingCondition );
}


/* ==============  G E N E R A L   S U B R O U T I N E S  =============== */


static int FindParameterID(char key, int number)
{
  int id;

  for(id=First_Parameter; id<=Last_Parameter; id++)
    {
    if( Key_(id) == key   &&   Number_(id) == number )
      {
      return( id );
      }
    }
  return(0);
}


/*-------------------------------------------------------------------*/


static int ValidKey(char key)
{
  int id;

  for(id=First_Parameter; id<=Last_Parameter; id++)
    {
    if( Key_(id) == key )
      {
      return( YES );
      }
    }
  return( NO );
}


/*-------------------------------------------------------------------*/


extern void SetDefaultValuesForParameters()
{
  int id;

  for(id=First_Parameter; id<=Last_Parameter; id++)
    {
    SetValue_( id, Default_(id) );
    }
}


/* ==============  P R I N T I N G	 R O U T I N E S  =============== */


static void PrintValue(int id, int value)
{
  char s[40];

  switch(Type_(id))
    {
    case BOOL: strcpy(s, value ? "YES" : "NO" );   break;
    case ENUM: strcpy(s, NameOfValue_(id,value));  break;
    case INT:  if( value == NOT_DEFINED )
                 {
                 strcpy(s, "NOT DEFINED");
                 }
               else
                 {
		 sprintf(s, "%i" , value);
                 }
               break;
    }
  printf("%s", s);
}


/*-------------------------------------------------------------------*/


static void PrintSyntax(char ch)
{
  if( !ValidKey( ch ))
    {
    return;
    }
  if( FindParameterID( ch, 0 ))      /* Non-parametric option */
    {
    printf("  %c%c               = ", OPTION_SYMBOL, ch);
    }
  else                               /* Apparently parametric option */
    {
    printf("  %c%c[n1,n2,...]    = ", OPTION_SYMBOL, ch);
    }
}


/*-------------------------------------------------------------------*/


static void PrintEnumList(int id)
{
  int i;
  int def = Default_(id);

  printf("\n");
  for(i=Min_(id); i<=Max_(id); i++)
    {
    printf("                            ");
    printf("%1i = ", i);
    PrintValue(id,i);
    if( def==i )
      {
      printf(" (default)");
      }
    printf("\n");
    }
}


/*-------------------------------------------------------------------*/


static void PrintDefault(int id)
{
  printf(" (default=");
  PrintValue(id, Default_(id));
  printf(")\n");
}


/*-------------------------------------------------------------------*/


static void PrintDocumentation(int id, int number)
{
  if( number )
    {
    printf("                      n%1i: ", number);
    }
  printf("%s", Document_(id));
  switch( Type_(id) )
    {
    case BOOL: PrintDefault(id);	break;
    case ENUM: PrintEnumList(id);	break;
    case INT:  PrintDefault(id);	break;
    }
}


/*-------------------------------------------------------------------*/


extern void PrintOptions()
{
  char ch;
  int  i, id;
  int  number;

  for(ch='A'; ch<='Z'; ch++)
    {
    PrintSyntax( ch );
    number = 0;
    for(i=0; i<=10; i++)
      {
      id = FindParameterID( ch, i );  /* Non-parametric option */
      if( id )
        {
        PrintDocumentation( id, number++ );
        }
      }
    }
}


/*-------------------------------------------------------------------*/


extern void PrintSelectedOptions()
{
  int id;

  for(id=First_Parameter; id<=Last_Parameter; id++)
    {
    if(ShallWePrint_(id))
      {
      printf("%-25s = ", Document_(id));
      PrintValue( id, Value_(id) );
      printf("\n");
      }
    }
}


/* ================  F I L E   O P E R A T I O N S  ================= */


void ReadOptions(FILE* f)
{
  int id;

  for(id=First_Parameter; id<=Last_Parameter; id++)
    {
    if( SaveToFile_(id) )
      {
      SetValue_(id, getw(f));
      }
    }
}


/*-------------------------------------------------------------------*/


void WriteOptions(FILE* f)
{
  int id;

  for(id=First_Parameter; id<=Last_Parameter; id++)
    {
    if( SaveToFile_(id) )
      {
      putw(Value_(id), f);
      }
    }
}


/* ==============  P A R A M E T E R   P A R S I N G  =============== */


static void ParseSubParameters(char* s,
                               int   limit,
                               int*  parsed,
                               int   values[])
{
  *parsed = 0;
  while( s && *s && *parsed < limit-1 )
    {
    switch( *s )
      {
      case ',': values[++(*parsed)] = NOT_DEFINED;
                s++;
                break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9': values[++(*parsed)] = abs(atoi(s));
                if( (s = strchr(s, ',')) != NULL )
                  {
                  s++;
                  }
                break;
      default:  s++;
                break;
      }
    }
}


/*-------------------------------------------------------------------*/


static void SetParameterValue(int id, int value)
{
/*printf("SetValue: id=%i, value=%i, type=%i.\n", id, value, Type_(id));*/
  switch( Type_( id ))
    {
    case BOOL:
      {
      SetValue_(id, YES);
      break;
      }
    case ENUM:
    case INT:
      {
      if( (value >= Min_(id))  &&  (value <= Max_(id)) )
        {
        SetValue_(id, value);
        }
      break;
      }
    } /* end switch */
}


/*-------------------------------------------------------------------*/


extern void ParseOption(char* s)
{
  int SubPar[20] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
                     -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
  int   parsed = 0;
  int   found  = NO;
  int   value;
  int   i, id;
  char  key;

  key = toupper( s[1] );
  ParseSubParameters(s + 2, 20, &parsed, SubPar);

  /* Non-parametric option */
  id = FindParameterID( key, 0 );
  if( id )
    {
    SetParameterValue( id, YES );
    found = YES;
    }

  /* Parametric option */
  for(i=1; i<=parsed; i++)
    {
    value = SubPar[i];
    id = FindParameterID( key, i );
    if( id )
      {
      SetParameterValue( id, value );
      found = YES;
      }
    }
if( !found )
  {
  printf("WARNING: Unknown option %c%c found.\n", OPTION_SYMBOL, key);
  }
}


